@@ -7,6 +7,7 @@ import numpy as np
 import matplotlib.pyplot as plt
 
 
+
 def getAtomsToMove(i,j,j_,ToBeMove,neighbors,ring=False):
     ToBeMove.append(j_)
     for n in neighbors[j_]:
@@ -18,6 +19,25 @@ def getAtomsToMove(i,j,j_,ToBeMove,neighbors,ring=False):
     return ToBeMove,ring
 
 
+def get_group(i,j,r,atoms):
+    positions = atoms.get_positions()
+    center    = 0.5*(positions[i] + positions[j]) 
+    vij = r[i][j]
+    rij = np.sqrt(np.sum(np.square(vij)))
+    vij = vij/rij
+
+    group_i,group_j = [],[]
+
+    for i_,x in enumerate(positions):
+        v    = x - center
+        dot_ = np.dot(v,vij)
+        if dot_<=0.0:
+           group_i.append(i_)
+        else:
+           group_j.append(i_)
+    return group_i,group_j
+
+
 def getNeighbor(natom,r,rcut,bo,botol=0.0):
     neighbors = [[] for _ in range(natom)]
     for i in range(natom-1):
@@ -469,8 +489,8 @@ class AtomDance(object):
                 sign = -1.0
              self.BondDistrubed.append(bd)
              atoms = self.set_bond_momenta(i,j,atoms,sign=sign)
-             return atoms
-      return atoms
+             return atoms,bd
+      return atoms,None
 
   def set_bond_momenta(self,i,j,atoms,sign=1.0):
       ha      = int(0.5*self.natom)
@@ -486,8 +506,7 @@ class AtomDance(object):
       ig      = len(group_i)
 
       if ring:
-         group_i = [i]
-         group_j = [j]
+         group_i,group_j = get_group(i,j,self.ir.r,atoms)
 
       vij   = self.ir.vr[j][i]/self.ir.r[i][j]
       massi = 0.0
@@ -535,6 +554,7 @@ class AtomDance(object):
 
       his     = TrajectoryWriter(traj,mode='w')
       images  = []
+      nb_ = 0
       for i_ in range(relax_step_):
           zmat_ = zmatrix+dz*(i_+1)
           zmat_ = np.where(zmat_>180.0,zmat_-360.0,zmat_)           #  scale to a reasonalbale range
@@ -549,10 +569,22 @@ class AtomDance(object):
           his.write(atoms=atoms)
           images.append(atoms)
           bonds   = getBonds(self.natom,self.ir.r,self.rmax*self.ir.re,self.ir.bo0)
-          newbond = self.checkBond(bonds)
+          newbond,nbd = self.checkBond(bonds)
           if newbond: 
-             relaxlog += 'stop at %d/%d  because new bond formed ...\n' %(i_,nbin)
-             break
+             iatom,jatom = nbd
+             if nb_ == 0:
+                nbd_ = nbd
+                r_   = self.ir.r[iatom][jatom]
+             else:
+                if nbd==nbd_:
+                   if self.ir.r[iatom][jatom]<r_:
+                      relaxlog += 'stop at %d/%d  because new bond formed ...\n' %(i_,nbin)
+                      break
+                   r_ = self.ir.r[iatom][jatom]
+                else:
+                   relaxlog += 'stop at %d/%d  because new bond formed ...\n' %(i_,nbin)
+                   break
+             nb_ += 1
       his.close()
       return images,relaxlog
 
@@ -752,12 +784,13 @@ class AtomDance(object):
 
   def checkBond(self,bonds):
       newbond = False
+      bd      = None
       for bd in bonds:
           bd_ = (bd[1],bd[0])
           if (bd not in self.InitBonds) and (bd_ not in self.InitBonds):
              newbond = True
-             return newbond 
-      return newbond
+             return newbond,bd 
+      return newbond,bd
 
   def check(self,wcheck=2,i=0,atoms=None,rmin=None):
       if atoms is None:
@@ -872,17 +905,17 @@ class AtomDance(object):
                  neighbors = getNeighbor(self.natom,self.ir.r,self.ir.r_cuta,self.ir.bo0)
       return atoms
 
-  def bend(self,ang=None,rang=20.0,nbin=10,scale=1.2,wtraj=False):
+  def bend(self,ang=None,rang=20.0,nbin=10,scale=1.2,traj='md.traj'):
       i,j,k = ang
       axis = [i,k]
-      images = self.rotate(atms=[i,k],axis=axis,o=j,rang=rang,nbin=nbin,wtraj=wtraj,scale=scale)
+      images = self.rotate(atms=[i,k],axis=axis,o=j,rang=rang,nbin=nbin,traj=traj,scale=scale)
       return images
 
-  def bend_axis(self,axis=None,group=None,rang=20,nbin=30,scale=1.2,wtraj=False):
-      images = self.rotate(atms=group,axis=axis,o=axis[0],rang=rang,nbin=nbin,wtraj=wtraj,scale=scale)
+  def bend_axis(self,axis=None,group=None,rang=20,nbin=30,scale=1.2,traj=None):
+      images = self.rotate(atms=group,axis=axis,o=axis[0],rang=rang,nbin=nbin,traj=traj,scale=scale)
       return images
 
-  def swing_group(self,ang=None,group=None,rang=20,nbin=30,scale=1.2,wtraj=False):
+  def swing_group(self,ang=None,group=None,rang=20,nbin=30,scale=1.2,traj=None):
       i,j,k = ang
       atoms = self.ir.atoms
       self.ir.calculate_Delta(atoms)
@@ -900,7 +933,7 @@ class AtomDance(object):
       else:
          uk  = uk/rk       
       images = self.rotate(atms=group,axis_vector=uk,o=j,rang=rang,nbin=nbin,
-                            wtraj=wtraj,scale=scale)
+                            traj=traj,scale=scale)
       return images
 
   def rotate(self,atms=None,axis=None,axis_vector=None,o=None,rang=20.0,nbin=10,traj=None,scale=1.2):
@@ -910,7 +943,8 @@ class AtomDance(object):
       # neighbors = getNeighbor(self.natom,self.ir.r,scale*self.ir.re,self.ir.bo0)
 
       images = []
-      # if not traj is None: his = TrajectoryWriter(traj,mode='a')
+      if not traj is None: his = TrajectoryWriter(traj,mode='w')
+
       if axis_vector is None:
          i,j   = axis
          vaxis = atoms.positions[j] - atoms.positions[i] 
@@ -919,7 +953,7 @@ class AtomDance(object):
          uk    = axis_vector
 
       a_     = -rang
-      while a_<rang:
+      for i in range(nbin):
           atoms_ = atoms.copy()
           for atomk in atms:
               vo  = atoms.positions[atomk] - atoms.positions[o] 
@@ -942,19 +976,20 @@ class AtomDance(object):
               atoms_.set_calculator(calc)
 
           images.append(atoms_)
-          if not traj is None: traj.write(atoms=atoms_)
+          if not traj is None: his.write(atoms=atoms_)
           a_ += da
-      # if not traj is None: his.close()
+
+      if not traj is None: his.close()
       return images
 
-  def swing(self,ang,st=60.0,ed=180.0,nbin=50,scale=1.2,wtraj=False):
-      da = (ed - st)/nbin
+  def swing(self,ang,st=60.0,ed=180.0,nbin=50,scale=1.2,traj=None):
+      da    = (ed - st)/nbin
       i,j,k = ang
       atoms = self.ir.atoms
       self.ir.calculate_Delta(atoms)
       # neighbors = getNeighbor(self.natom,self.ir.r,scale*self.ir.re,self.ir.bo0)
       images = []
-      if wtraj: his = TrajectoryWriter('swing.traj',mode='w')
+      if not traj is None: his = TrajectoryWriter(traj,mode='w')
 
       vij = atoms.positions[i] - atoms.positions[j] 
       vjk = atoms.positions[k] - atoms.positions[j]
@@ -972,28 +1007,76 @@ class AtomDance(object):
       uj = np.cross(uk,ui)
       a_ = st
 
-      while a_<ed:
-            atoms_ = atoms.copy()
-            a = a_*3.141593/180.0
-            p = r*np.cos(a)*ui + r*np.sin(a)*uj
-            atoms_.positions[k] = atoms_.positions[j]+p
-            self.ir.calculate(atoms_)
+      for i in range(nbin):
+          atoms_ = atoms.copy()
+          a = a_*3.141593/180.0
+          p = r*np.cos(a)*ui + r*np.sin(a)*uj
+          atoms_.positions[k] = atoms_.positions[j]+p
+          self.ir.calculate(atoms_)
 
-            calc = SinglePointCalculator(atoms_,energy=self.ir.E)
-            atoms_.set_calculator(calc)
+          calc = SinglePointCalculator(atoms_,energy=self.ir.E)
+          atoms_.set_calculator(calc)
 
-            images.append(atoms_)
-            if wtraj: his.write(atoms=atoms_)
-            a_ += da
+          images.append(atoms_)
+          if not traj is None: his.write(atoms=atoms_)
+          a_ += da
 
-      if wtraj: his.close()
+      if not traj is None: his.close()
       return images
 
-  def stretch(self,pair,atoms=None,nbin=20,st=0.7,ed=1.3,scale=1.25,traj=None,ToBeMoved=None):
+  def scan_pes(self,i,n,neighbor,atoms,images,r,nbin=[5],dr=[0.1]):
+      if n>=len(neighbor):
+         return None
+      k = neighbor[n]
+
+      if n>=len(nbin):
+         nb_ = nbin[0]
+      else:
+         nb_ = nbin[n]
+
+      if n>=len(dr):
+         dr_ = dr[0]
+      else:
+         dr_ = dr[n]
+
+      imag_ = self.stretch([i,k],atoms=atoms,nbin=nb_,
+                           rst=r[k]-dr_,
+                           red=r[k]+dr_,
+                           neighbors=self.neighbors)
+      for atoms_ in imag_:
+          imag = self.scan_pes(i,n+1,neighbor,atoms_,images,r,nbin=nbin,dr=dr)
+          if imag is None:
+             images.append(atoms_)
+      return images
+
+
+  def pes(self,i,atoms,neighbor=None,nbin=[5],scandr=[0.2],traj=None):
+      images = []
+      r      = {}
+
+      if neighbor is None:
+         neighbor = self.neighbors[i]
+
+      for j_ in neighbor:
+          r[j_]   = self.ir.r[i][j_]  
+      images = self.scan_pes(i,0,neighbor,atoms,images,r,nbin=nbin,dr=scandr)
+
+      if not traj is None:
+         his = TrajectoryWriter(traj,mode='w')
+         for atoms in images:
+             his.write(atoms=atoms)
+         his.close()
+       
+      return images
+  
+  def stretch(self,pair,atoms=None,nbin=20,rst=0.7,red=1.3,scale=1.25,traj=None,
+              ToBeMoved=None,neighbors=None):
       if atoms is None:
          atoms = self.ir.atoms
-      self.ir.calculate_Delta(atoms)
-      neighbors = getNeighbor(self.natom,self.ir.r,scale*self.ir.re,self.ir.bo0)
+
+      if neighbors is None:
+         self.ir.calculate_Delta(atoms)
+         neighbors = getNeighbor(self.natom,self.ir.r,scale*self.ir.re,self.ir.bo0)
       images = []
  
       if not traj is None: his = TrajectoryWriter(traj,mode='w')
@@ -1004,16 +1087,18 @@ class AtomDance(object):
          ToBeMove,ring = getAtomsToMove(i,j,j,ToBeMove,neighbors)
       else:
          ToBeMove = ToBeMoved
-
-      bin_     = (self.ir.re[i][j]*ed - self.ir.re[i][j]*st)/nbin
-      moveDirection = self.ir.vr[j][i]/self.ir.r[i][j]
-
+         ring     = False
+      
       if ring:
-         return None
+         # return None
+         ToBeMove = [j]
+
+      bin_     = (red - rst)/nbin
+      moveDirection = self.ir.vr[j][i]/self.ir.r[i][j]
 
       for n in range(nbin):
           atoms_ = atoms.copy()
-          moveV  = atoms.positions[i] + moveDirection*(self.ir.re[i][j]*st+bin_*n)-atoms.positions[j]
+          moveV  = atoms.positions[i] + moveDirection*(rst+bin_*n)-atoms.positions[j]
           # print(self.ir.re[i][j]*self.rmin+bin_*n)
           for m in ToBeMove:
               # sPos   = atoms.positions[i] + self.ir.re[i][m]*self.rmin*moveDirection
@@ -1042,8 +1127,10 @@ class AtomDance(object):
          zfile = self.label+'.zmat'
       with open (zfile,'w') as f:
            for i,iatom in enumerate(self.zmat_id):
-               print('%3d %3d %3d %3d' %(iatom,self.zmat_index[i][0],self.zmat_index[i][1],self.zmat_index[i][2]),
-                     ' %7.4f  %8.4f  %9.4f' %(zmatrix[i][0],zmatrix[i][1],zmatrix[i][2]),
+               i_ = self.zmat_id[i]
+               print('[ \'%s\',%3d, %3d, %3d, %3d,' %(self.atom_name[i_],iatom,self.zmat_index[i][0],
+                                              self.zmat_index[i][1],self.zmat_index[i][2]),
+                     ' %7.4f,%8.4f,%9.4f ],' %(zmatrix[i][0],zmatrix[i][1],zmatrix[i][2]),
                      file=f)
 
   def close(self):
