@@ -52,63 +52,36 @@ def get_zpe(p,atom_name):
 class LearningMachine(object):
   ''' recursive learning machine constructed by I-ReaxFF and siesta '''
   def __init__(self,initConfig='poscar.gen',
-               direcs=None,
-               ncpu=40,
+               direcs=None,ncpu=40,
                maxiter=1000,
-               dt_mlmd=0.1,
-               dt_aimd=0.3,
-               max_batch=10,
-               dft_step=5,
-               batch=50,
-               step=20000,
-               md_step=20,
-               mom_step=100,
-               MaxMDstep=500,
-               mpopt=1,
-               messages=1,
-               T=300,
-               Tmax=1000,
+               dt_mlmd=0.1, dt_aimd=0.3,dft_step=5,
+               batch=50,max_batch=10,
+               step=20000,md_step=20,MaxMDstep=500,colFrame=10,
+               mom_step=100,beta=0.777,
+               mpopt=1, messages=1,
+               optword='nocoul',
+               T=300,Tmax=1000,
                label=None,
-               convergence=0.01,
-               nconvergence=3,
-               accCriteria=0.95,
-               lossCriteria=1000.0,
-               lossTole=1.0,
-               accTole=0.005,
-               accMax=0.96,
-               accInc=1.0,
-               resetAcc=5,
+               convergence=0.01,nconvergence=3,
+               accCriteria=0.95,lossCriteria=1000.0,lossTole=1.0,accTole=0.005,
+               accMax=0.96,accInc=1.0,mdInc=1.2,resetAcc=5,
                learnWay=5,
-               FreeAtoms=None,
-               FirstAtom=None,
-               colFrame=10,
-               mdInc=1.2,
+               FreeAtoms=None,FirstAtom=None,
                rodic=None,
-               rmin=0.6,
-               rmax=1.3,
-               angmax=15.0,
+               rmin=0.6,rmax=1.3,angmax=15.0,
                UseZmat=True,
-               EngTole=0.05,
-               dEtole=0.2,
-               d2Etole=0.05,
-               dEstop=1.0,
+               EngTole=0.05,dEtole=0.2,d2Etole=0.05,dEstop=2.0,
                nn=True,vdwnn=False,
-               bo_layer=[2,1],
-               mf_layer=[6,1],
-               be_layer=[6,1],
-               EnergyFunction=3,
-               MessageFunction=1,
+               bo_layer=[2,1],mf_layer=[6,1],be_layer=[6,1],
+               EnergyFunction=1,MessageFunction=3,
                bom={'others':1.0},
                bore={'others':0.45},
                weight={'others':2.0},
-               spv_bm=False,
-               spv_be=False,
+               spv_bm=False,spv_be=False,
                bo_penalty=10000.0,
                mefac=0.1,
                writelib=5000,
-               xcf='VDW',
-               xca='DRSLL',
-               basistype='DZP'):
+               xcf='VDW',xca='DRSLL',basistype='DZP'):
       ''' max_batch: max number of batch 
               batch: batch size
       '''
@@ -125,7 +98,7 @@ class LearningMachine(object):
       self.rmin         = rmin       # minimum bond length used to supervise ML-MD simulation
       self.rmax         = rmax
       self.angmax       = angmax
-      self.UseZmat    = UseZmat
+      self.UseZmat      = UseZmat
       self.convergence  = convergence
       self.nconvergence = nconvergence
       self.learnWay     = learnWay    # the way of learing
@@ -145,6 +118,7 @@ class LearningMachine(object):
       self.Tmax         = Tmax
       self.md_step      = md_step
       self.mom_step     = mom_step
+      self.beta         = beta
       self.colFrame     = colFrame
       self.EngTole      = EngTole
       self.dEtole       = dEtole
@@ -152,6 +126,7 @@ class LearningMachine(object):
       self.dEstop       = dEstop
       self.MaxMDstep    = MaxMDstep
       self.mpopt        = mpopt
+      self.optword      = optword
       self.messages     = messages
       self.mdInc        = mdInc       # MD step increase factor
       self.nn           = nn
@@ -238,15 +213,17 @@ class LearningMachine(object):
          ms        = len(self.freeatoms)
       else:
          ms        = 0
+
+      self.learnpair = None
       self.save_config()
-      converg      = False
-      nconvergence = 0
+      converg        = False
+      nconvergence   = 0
       unreasonable_zv,uzv_ = None,None
-      zmat_variable= None
-      zmatopt      = None
-      zmats        = None
-      zvlo,zvhi    = 0.0,0.0
-      u_zvlo,u_zvhi= 0.0,0.0
+      zmat_variable  = None
+      zmatopt        = None
+      zmats          = None
+      zvlo,zvhi      = 0.0,0.0
+      u_zvlo,u_zvhi  = 0.0,0.0
 
       while running:
           optlog   = None
@@ -267,12 +244,14 @@ class LearningMachine(object):
              atoms = read(self.initConfig)
              learnWay = 1
           
-          if learnWay==4 or learnWay==5:
+          if learnWay in [4,5,6]:
              momsteps += mdsteps
              if momsteps >= mom_step:
                 momsteps  = 0
                 if learnWay == 4:
-                   atoms     = self.a.bond_momenta(atoms)
+                   atoms,self.learnpair = self.a.bond_momenta(atoms)
+                   if self.learnpair is None:
+                      learnWay = 3
                 elif learnWay == 5:
                    if ms<self.natom:
                       i = self.a.zmat_id[ms]
@@ -291,15 +270,21 @@ class LearningMachine(object):
                       self.save_config()
                       if ms+1<self.natom:
                          j = self.a.zmat_id[ms+1]
+                         self.learnpair = [i,j]
                          atoms = self.a.set_bond_momenta(i,j,atoms)
-                   learnWay = 4
-                learnWay  = 2
+                      else:
+                         learnWay = 3
+                         self.learnpair = None
+                   if learnWay ==5: learnWay = 4
+                # learnWay  = 2
                 ms       += 1
-          self.get_fixatom()
-          atoms.set_constraint(self.fixatoms)
+
+          # self.get_fixatom()
+          # atoms.set_constraint(self.fixatoms)
+
           atoms.write('poscar.gen')                                    # for aimd
 
-          if learnWay==1 : 
+          if learnWay==1: 
              dft_step = self.dft_step
           else:
              dft_step = int(mdsteps/self.colFrame)+1
@@ -328,7 +313,7 @@ class LearningMachine(object):
                                                      convergence=self.accCriteria,
                                                      lossConvergence=self.lossCriteria,
                                                      nn=self.nn,vdwnn=self.vdwnn,
-                                                     mpopt=self.mpopt,
+                                                     mpopt=self.mpopt,optword=self.optword,
                                                      bo_layer=self.bo_layer,
                                                      mf_layer=self.mf_layer,
                                                      be_layer=self.be_layer,
@@ -357,21 +342,29 @@ class LearningMachine(object):
           zmats   = None
           images  = None
           zmatopt = zmat_variable
-          if Deformed>=1.0:
-             images,relaxlog = self.a.zmat_relax(atoms=atoms,zmat_variable=zmat_variable,nbin=mdsteps,
-                                                 zvlo=zvlo,zvhi=zvhi,traj='md.traj')
-          elif not unreasonable_zv is None:
-             images,relaxlog = self.a.zmat_relax(atoms=atoms,zmat_variable=unreasonable_zv,nbin=mdsteps,
-                                                 zvlo=u_zvlo,zvhi=u_zvhi,traj='md.traj')
+          if learnWay==6:
+             if ms<len(self.freeatoms):
+                i = self.freeatoms[ms]
+                self.a.pes(i,atoms,nbin=dft_step,dr=0.1,traj='md.traj')
+             if ms == len(self.freeatoms) -1:
+             	self.learnWay=3
           else:
-             e_gmd,mdsteps,Deformed,zmats,zmat_variable,zvlo,zvhi = self.mlmd(cwd,run_dir,atoms, 10000,iter_,learnWay) 
-             if self.UseZmat and mdsteps<=2:
-                mdsteps= 7
-                images,relaxlog = self.a.zmat_relax(atoms=atoms,nbin=20,relax_step=mdsteps,traj='md.traj')
-          if not images is None:
-             Deformed,zmat,zmat_variable,zvlo,zvhi = check_zmat(atoms=images[-1],rmin=self.rmin,rmax=self.rmax,
-                                    angmax=self.angmax,zmat_id=self.a.zmat_id,
-                                    zmat_index=self.a.zmat_index,InitZmat=self.a.InitZmat)
+             if Deformed>=1.0:
+                images,relaxlog = self.a.zmat_relax(atoms=atoms,zmat_variable=zmat_variable,nbin=mdsteps,
+                                                    zvlo=zvlo,zvhi=zvhi,traj='md.traj')
+             elif not unreasonable_zv is None:
+                images,relaxlog = self.a.zmat_relax(atoms=atoms,zmat_variable=unreasonable_zv,nbin=mdsteps,
+                                                    zvlo=u_zvlo,zvhi=u_zvhi,traj='md.traj')
+             else:
+                e_gmd,mdsteps,Deformed,zmats,zmat_variable,zvlo,zvhi = self.mlmd(cwd,run_dir,atoms, 10000,
+                                                iter_,learnWay,beta=self.beta,learnpair=self.learnpair) 
+                if self.UseZmat and mdsteps<=2:
+                   mdsteps= 7
+                   images,relaxlog = self.a.zmat_relax(atoms=atoms,nbin=20,relax_step=mdsteps,traj='md.traj')
+             if not images is None:
+                Deformed,zmat,zmat_variable,zvlo,zvhi = check_zmat(atoms=images[-1],rmin=self.rmin,
+                                       rmax=self.rmax,angmax=self.angmax,zmat_id=self.a.zmat_id,
+                                       zmat_index=self.a.zmat_index,InitZmat=self.a.InitZmat)
 
           step += mdsteps                                          # run ML-MD test training results     
 
@@ -390,7 +383,7 @@ class LearningMachine(object):
 
           self.save_config()
 
-          if learnWay==3 or learnWay==4 or learnWay==5: e_gmd = eml_
+          if learnWay in [3,4,5]: e_gmd = eml_
           e_gulp.append(e_gmd[0])
           diff  = abs(e_gmd[0]-e_aimd[0])
           it.append(iter_)
@@ -472,16 +465,16 @@ class LearningMachine(object):
       chdir(cwd)
       return eaimd,emlmd,dEmax,d2Emax
 
-  def mlmd(self,cwd,run_dir,atoms,Tmax,Iter,learnWay):        # run classic MD to test training results
+  def mlmd(self,cwd,run_dir,atoms,Tmax,Iter,learnWay,learnpair=None,beta=None):        
+      ''' run classic MD to test training results '''
       mdstep = max(int(self.md_step/5),2) if learnWay==2 else self.md_step
-      irmd = IRMD(atoms=atoms,label=self.label,
+      irmd = IRMD(atoms=atoms,label=self.label,Iter=Iter,initT=self.T,
                   time_step=self.dt_mlmd,totstep=mdstep,Tmax=Tmax,
-                  Iter=Iter,initT=self.T,
                   ro=self.ro,rmin=self.rmin,rmax=self.rmax,angmax=self.angmax,
                   UseZmat=self.UseZmat,InitZmat=self.a.InitZmat,
                   zmat_id=self.a.zmat_id,zmat_index=self.a.zmat_index,
-                  dEstop=self.dEstop,
-                  nn=self.nn,vdwnn=self.vdwnn)
+                  dEstop=self.dEstop,nn=self.nn,vdwnn=self.vdwnn,
+                  learnpair=learnpair,beta=beta)
       if learnWay==2:
          Deformed,zmats,zv,zvlo,zvhi = irmd.opt()
       else:
@@ -543,6 +536,7 @@ class LearningMachine(object):
                     'dEtole':self.dEtole,
                     'd2Etole':self.d2Etole,
                     'dEstop':self.dEstop,
+                    'beta':self.beta,
                     'mdInc':self.mdInc,
                     'dt_aimd':self.dt_aimd,
                     'dt_mlmd':self.dt_mlmd,
@@ -552,6 +546,7 @@ class LearningMachine(object):
                     'maxiter':self.maxiter,
                     'max_batch':self.max_batch,
                     'mpopt':self.mpopt,
+                    'optword':self.optword,
                     'mefac':self.mefac,
                     'learnWay':self.learnWay,
                     'colFrame':self.colFrame,
@@ -582,6 +577,7 @@ class LearningMachine(object):
            self.dEtole       = options['dEtole'] 
            self.d2Etole      = options['d2Etole'] 
            self.dEstop       = options['dEstop'] 
+           self.beta         = options['beta'] 
            self.mdInc        = options['mdInc'] 
            self.dt_aimd      = options['dt_aimd'] 
            self.dt_mlmd      = options['dt_mlmd'] 
@@ -591,6 +587,7 @@ class LearningMachine(object):
            self.maxiter      = options['maxiter']
            self.max_batch    = options['max_batch']
            self.mpopt        = options['mpopt']
+           self.optword      = options['optword']
            self.mefac        = options['mefac']
            self.learnWay     = options['learnWay']
            self.colFrame     = options['colFrame']
