@@ -82,7 +82,6 @@ class IRFF(Calculator):
   def __init__(self,atoms=None,
                mol=None,
                libfile='ffield',
-               rcut=None,rcuta=None,
                vdwcut=10.0,
                atol=0.001,
                hbtol=0.001,
@@ -120,6 +119,8 @@ class IRFF(Calculator):
          self.mf_layer       = j['mf_layer']
          self.be_layer       = j['be_layer']
          self.vdw_layer      = j['vdw_layer']
+         rcut                = j['rcut']
+         rcuta               = j['rcutAngle']
          lf.close()
          self.init_bonds()
          if mol is None:
@@ -136,7 +137,9 @@ class IRFF(Calculator):
                        read_lib(libfile=libfile,zpe=False)
          m             = None
          self.bo_layer = None
-         self.emol      = 0.0
+         self.emol     = 0.0
+         rcut          = None
+         rcuta         = None
 
       if m is None:
          self.nn=False
@@ -218,10 +221,8 @@ class IRFF(Calculator):
 
   def set_rcut(self,rcut,rcuta): 
       rcut_,rcuta_,re_ = setRcut(self.bonds,rcut,rcuta,None)
-      if rcut is None:  ## bond order compute cutoff
-         self.rcut = rcut_
-      if rcuta is None: ## angle term cutoff
-         self.rcuta = rcuta_
+      self.rcut  = rcut_    ## bond order compute cutoff
+      self.rcuta = rcuta_   ## angle term cutoff
 
       # self.r_cut = np.zeros([self.natom,self.natom],dtype=np.float32)
       # self.r_cuta = np.zeros([self.natom,self.natom],dtype=np.float32)
@@ -368,19 +369,26 @@ class IRFF(Calculator):
 
           Fi    = self.f_nn('f'+str(t),[Dbj,Dbi,self.H[t-1]],layer=self.mf_layer[1])
           Fj    = torch.transpose(Fi,0,1)
-          F     = 2.0*Fi*Fj
+
 
           if self.MessageFunction==1:
+             F   = 2.0*Fi*Fj
              self.Hsi.append(self.Hsi[t-1]*F)
              self.Hpi.append(self.Hpi[t-1]*F)
              self.Hpp.append(self.Hpp[t-1]*F)
           elif self.MessageFunction==2:
+             F   = Fi*Fj
              Fsi = F[:,:,0]
              Fpi = F[:,:,1]
              Fpp = F[:,:,2]
              self.Hsi.append(self.Hsi[t-1]*Fsi)
              self.Hpi.append(self.Hpi[t-1]*Fpi)
              self.Hpp.append(self.Hpp[t-1]*Fpp)
+          elif self.MessageFunction==3:
+             F   = Fi*Fj
+             self.Hsi.append(self.Hsi[t-1]*F)
+             self.Hpi.append(self.Hpi[t-1]*F)
+             self.Hpp.append(self.Hpp[t-1]*F)
           self.H.append(self.Hsi[t]+self.Hpi[t]+self.Hpp[t])
           self.D.append(torch.sum(self.H[t],1) )
 
@@ -404,16 +412,13 @@ class IRFF(Calculator):
 
       if self.EnergyFunction==3: 
          e_ = self.f_nn('fe',[self.bosi,self.bopi,self.bopp],layer=self.be_layer[1])
-         self.esi   = self.bo0*e_
+         self.esi = self.bo0*e_
       elif self.EnergyFunction==1:
-         Fsi  = self.f_nn('fesi',[self.bosi],layer=self.be_layer[1])
-         Fpi  = self.f_nn('fepi',[self.bopi],layer=self.be_layer[1])
-         Fpp  = self.f_nn('fepp',[self.bopp],layer=self.be_layer[1])
-         self.esi = self.bosi*Fsi + self.bopi*Fpi + self.bopp*Fpp
+         self.esi = self.f_nn('fe',[self.bosi,self.bopi,self.bopp],layer=self.be_layer[1])
       elif self.EnergyFunction==2:
          Fi = self.f_nn('fe',[Dbj,Dbi,self.bosi],layer=self.be_layer[1])
          Fj = torch.transpose(Fi,0,1)
-         F  = 2.0*Fi*Fj
+         F  = Fi*Fj
          self.esi   = self.bosi*F
 
 
@@ -1111,10 +1116,10 @@ class IRFF(Calculator):
 
   def set_m(self,m):
       self.m = {}
-      if self.EnergyFunction==1:
-         pres = ['fesi','fepi','fepp','fsi','fpi','fpp','fv']
-      else:
-         pres = ['fe','fsi','fpi','fpp','fv']
+      # if self.EnergyFunction==1:
+      #    pres = ['fesi','fepi','fepp','fsi','fpi','fpp','fv']
+      # else:
+      pres = ['fe','fsi','fpi','fpp','fv']
       for t in range(1,self.messages+1):
           pres.append('f'+str(t))
 
