@@ -76,7 +76,6 @@ class IRFF_NP(object):
   implemented_properties = ["energy", "forces"] # , "stress"]
   def __init__(self,atoms=None,
                libfile='ffield',
-               rcut=None,rcuta=None,
                vdwcut=10.0,
                nn=False,vdwnn=False,
                messages=1,
@@ -110,8 +109,9 @@ class IRFF_NP(object):
          self.bo_layer = j['bo_layer']
          self.mf_layer = j['mf_layer']
          self.be_layer = j['be_layer']
-         self.vdw_layer = j['vdw_layer']
-         
+         self.vdw_layer= j['vdw_layer']
+         rcut          = j['rcut']
+         rcuta         = j['rcutAngle']
          lf.close()
          self.init_bonds()
       else:
@@ -119,6 +119,9 @@ class IRFF_NP(object):
                        read_lib(libfile=libfile,zpe=False)
          m                = None
          self.bo_layer    = None
+         rcut             = None
+         rcuta            = None
+
       if m is None:
          self.nn=False
 
@@ -211,10 +214,8 @@ class IRFF_NP(object):
 
   def set_rcut(self,rcut,rcuta): 
       rcut_,rcuta_,re_ = setRcut(self.bonds,rcut,rcuta,None)
-      if rcut is None:  ## bond order compute cutoff
-         self.rcut = rcut_
-      if rcuta is None: ## angle term cutoff
-         self.rcuta = rcuta_
+      self.rcut  = rcut_    ## bond order compute cutoff
+      self.rcuta = rcuta_  ## angle term cutoff
 
       self.r_cut = np.zeros([self.natom,self.natom],dtype=np.float32)
       self.r_cuta = np.zeros([self.natom,self.natom],dtype=np.float32)
@@ -364,7 +365,6 @@ class IRFF_NP(object):
           # self.Fi  = self.f_nn('f',[Di,Dj,Dbi,Dbj],layer=self.mf_layer[1])
           Fi    = self.f_nn('f'+str(t),[Dbj,Dbi,self.H[t-1]],layer=self.mf_layer[1])
 
-
           if self.MessageFunction==1:
              Fj    = np.transpose(Fi,[1,0])
              F     = 2.0*Fi*Fj
@@ -374,7 +374,7 @@ class IRFF_NP(object):
              self.Hpp.append(self.Hpp[t-1]*F)
           elif self.MessageFunction==2:
              Fj    = np.transpose(Fi,[1,0,2])
-             F     = 2.0*Fi*Fj
+             F     = Fi*Fj
              self.F.append(F)
              Fsi = F[:,:,0]
              Fpi = F[:,:,1]
@@ -382,6 +382,13 @@ class IRFF_NP(object):
              self.Hsi.append(self.Hsi[t-1]*Fsi)
              self.Hpi.append(self.Hpi[t-1]*Fpi)
              self.Hpp.append(self.Hpp[t-1]*Fpp)
+          elif self.MessageFunction==3:
+             Fj    = np.transpose(Fi,[1,0])
+             F     = Fi*Fj
+             self.F.append(F)
+             self.Hsi.append(self.Hsi[t-1]*F)
+             self.Hpi.append(self.Hpi[t-1]*F)
+             self.Hpp.append(self.Hpp[t-1]*F)
           self.H.append(self.Hsi[t]+self.Hpi[t]+self.Hpp[t])
           self.D.append(np.sum(self.H[t],axis=1))  
 
@@ -405,16 +412,13 @@ class IRFF_NP(object):
 
       if self.EnergyFunction==3: 
          e_ = self.f_nn('fe',[self.bosi,self.bopi,self.bopp],layer=self.be_layer[1])
-         self.esi   = self.bo0*e_
+         self.esi = self.bo0*e_
       elif self.EnergyFunction==1:
-         Fsi  = self.f_nn('fesi',[self.bosi],layer=self.be_layer[1])
-         Fpi  = self.f_nn('fepi',[self.bopi],layer=self.be_layer[1])
-         Fpp  = self.f_nn('fepp',[self.bopp],layer=self.be_layer[1])
-         self.esi   = self.bosi*Fsi + self.bopi*Fpi + self.bopp*Fpp
+         self.esi = self.f_nn('fe',[self.bosi,self.bopi,self.bopp],layer=self.be_layer[1])
       elif self.EnergyFunction==2:
          Fi = self.f_nn('fe',[Dbj,Dbi,self.bosi],layer=self.be_layer[1])
          Fj = np.transpose(Fi,[1,0])
-         F  = 2.0*Fi*Fj
+         F  = Fi*Fj
          self.esi   = self.bosi*F
 
 
@@ -464,10 +468,12 @@ class IRFF_NP(object):
       self.NLPOPT  = 0.5*(self.P['vale'] - self.P['val'])
       self.Delta_e = 0.5*(self.Delta - self.P['vale'])
       self.DE      = relu(-np.ceil(self.Delta_e))  # number of lone pair electron
+
       self.nlp     = self.DE + np.exp(-self.P['lp1']*4.0*np.square(1.0+self.Delta_e+self.DE))
       
       self.Delta_lp= self.NLPOPT-self.nlp   
       self.Dlp     = self.Delta - self.P['val'] - self.Delta_lp   
+
       self.Dpil    = np.sum(np.expand_dims(self.Dlp,axis=0)*(self.bopi+self.bopp),1)
 
       self.explp   = 1.0+np.exp(-self.P['lp3']*self.Delta_lp)
@@ -597,12 +603,12 @@ class IRFF_NP(object):
       Dk    = self.Delta[self.angk]
       self.expcoa1 = np.exp(self.P['coa2']*Dcoa)
 
-      texp0 = np.divide(self.P['coa1'],1.0+self.expcoa1)  
-      texp1 = np.exp(-self.P['coa3']*np.square(Di-boij))
-      texp2 = np.exp(-self.P['coa3']*np.square(Dk-bojk))
-      texp3 = np.exp(-self.P['coa4']*np.square(boij-1.5))
-      texp4 = np.exp(-self.P['coa4']*np.square(bojk-1.5))
-      self.etcon = texp0*texp1*texp2*texp3*texp4*self.fijk
+      self.texp0 = np.divide(self.P['coa1'],1.0+self.expcoa1)  
+      self.texp1 = np.exp(-self.P['coa3']*np.square(Di-boij))
+      self.texp2 = np.exp(-self.P['coa3']*np.square(Dk-bojk))
+      self.texp3 = np.exp(-self.P['coa4']*np.square(boij-1.5))
+      self.texp4 = np.exp(-self.P['coa4']*np.square(bojk-1.5))
+      self.etcon = self.texp0*self.texp1*self.texp2*self.texp3*self.texp4*self.fijk
       # print(texp0.shape)
       # print(self.etcon.shape)
       self.Etcon = np.sum(self.etcon)
@@ -1124,10 +1130,10 @@ class IRFF_NP(object):
 
   def set_m(self,m):
       self.m = {}
-      if self.EnergyFunction==1:
-         pres = ['fesi','fepi','fepp','fsi','fpi','fpp','fv']
-      else:
-         pres = ['fe','fsi','fpi','fpp','fv']
+      # if self.EnergyFunction==1:
+      #    pres = ['fesi','fepi','fepp','fsi','fpi','fpp','fv']
+      # else:
+      pres = ['fe','fsi','fpi','fpp','fv']
       for t in range(1,self.messages+1):
           pres.append('f'+str(t))
 
