@@ -13,6 +13,32 @@ from ase.io.trajectory import Trajectory
 from ase import units
 
 
+
+def get_pressure(atoms):           
+    ir = IRFF(atoms=atoms_md[-1],
+            libfile='ffield.json',
+            rcut=None,
+            nn=True,
+            CalStress=True)
+    p = []
+
+    for i,atoms in enumerate(atoms_md):
+        nonzero = 0
+
+        ir.calculate(atoms=atoms)
+        stress  = ir.results['stress']
+
+        for _ in range(3):
+          if abs(stress[0])>0.0000001:
+             nonzero += 1
+
+        pressure = (-stress[0] - stress[1] - stress[2])/nonzero
+        p.append(pressure*GPa)
+    V = atoms.get_volume()
+    # print('step %d' %i,pressure*GPa)
+    return p,V
+
+
 def fvr(x):
     xi  = np.expand_dims(x,axis=0)
     xj  = np.expand_dims(x,axis=1) 
@@ -41,8 +67,8 @@ class IRMD(object):
                initT=300,Tmax=10000,time_step=0.1,Iter=0,
                ro=None,rmin=0.6,rmax=1.3,angmax=20.0,
                UseZmat=False,zmat_id=None,zmat_index=None,InitZmat=None,
-               dEstop=0.5,
-               print_interval=1):
+               learnpair=None,beta=None,
+               dEstop=0.5,print_interval=1):
       self.Epot    = []
       self.epot    = 0.0
       self.ekin    = 0.0
@@ -64,6 +90,8 @@ class IRMD(object):
       self.InitZmat  = InitZmat
       self.zmats     = []
       self.dEstop    = dEstop
+      self.learnpair = learnpair
+      self.beta      = beta
       self.print_interval = print_interval
 
       if self.atoms is None:
@@ -91,10 +119,26 @@ class IRMD(object):
 
   def run(self):
       self.zv,self.zvlo,self.zvhi = None,0.0,0.0
-      self.dyn = VelocityVerlet(self.atoms, self.time_step*units.fs,trajectory='md.traj')  
+      self.dyn = VelocityVerlet(self.atoms, self.time_step*units.fs,trajectory='md.traj') 
+      if (not self.learnpair is None) and (not self.beta is None):
+         vij = self.atoms.positions[self.learnpair[1]] - self.atoms.positions[self.learnpair[0]]
+         rij = np.sqrt(np.sum(np.square(vij)))
+         vij = vij/rij
+
       def printenergy(a=self.atoms):
-          epot_      = a.get_potential_energy()
-          n,self.Deformed = 0,0
+          n             = 0
+          self.Deformed = 0.0
+          epot_         = a.get_potential_energy()
+
+          if (not self.learnpair is None) and (not self.beta is None):
+             v    = self.atoms.get_velocities()
+             # vv   = np.sqrt(np.sum(np.square(v),axis=1))
+             for iv,v_ in enumerate(v):
+                 d  = np.dot(v_,vij)
+                 vd = d*vij
+                 v[iv] = vd + self.beta*(v[iv]-vd)
+             self.atoms.set_velocities(v)
+
           if self.UseZmat:
              self.Deformed,zmat,self.zv,self.zvlo,self.zvhi = check_zmat(atoms=a,rmin=self.rmin,rmax=self.rmax,
                                              angmax=self.angmax,zmat_id=self.zmat_id,
@@ -124,9 +168,9 @@ class IRMD(object):
                             newbond    = self.checkBond(bonds)
              self.zmats.append(zmat)
           else:
-             r        = a.calc.r.detach().numpy()
-             i_       = np.where(np.logical_and(r<self.rmin*self.ro,r>0.0001))
-             n        = len(i_[0])
+             r    = a.calc.r.detach().numpy()
+             i_   = np.where(np.logical_and(r<self.rmin*self.ro,r>0.0001))
+             n    = len(i_[0])
 
           if len(self.Epot)==0:
              dE_ = 0.0
