@@ -34,13 +34,13 @@ class MPNN(ReaxFF):
                batch_size=200,sample='uniform',
                hbshort=6.75,hblong=7.5,
                vdwcut=10.0,
-               rcut=None,rcuta=None,re=None,
                bore={'C-C':0.5,'others':0.45},
                bom={'others':1.05},
                weight={'others':1.0},
                ro_scale=0.1,
                clip_op=True,
                InitCheck=True,
+               resetDeadNeuron=False,
                nn=True,
                messages=1,
                TwoBodyOnly=False,
@@ -96,8 +96,8 @@ class MPNN(ReaxFF):
                       optmol=optmol,mefac=mefac,
                       nanv=nanv,batch_size=batch_size,sample=sample,
                       hbshort=hbshort,hblong=hblong,vdwcut=vdwcut,
-                      rcut=rcut,rcuta=rcuta,re=re,ro_scale=ro_scale,
-                      clip_op=clip_op,InitCheck=InitCheck,nn=nn,
+                      ro_scale=ro_scale,
+                      clip_op=clip_op,InitCheck=InitCheck,resetDeadNeuron=resetDeadNeuron,nn=nn,
                       bo_layer=bo_layer,spec=spec,sort=sort,pkl=pkl,weight=weight,
                       bore=bore,bo_penalty=bo_penalty,
                       optMethod=optMethod,maxstep=maxstep,
@@ -115,7 +115,7 @@ class MPNN(ReaxFF):
   def supervise(self):
       ''' supervised learning term'''
       l_atol = 0.0
-      self.diffa,self.diffb,self.diffe = {},{},{}
+      self.diffa,self.diffb,self.diffe,self.diffesi = {},{},{},{}
       for bd in self.bonds: 
           [atomi,atomj] = bd.split('-') 
           if self.nbd[bd]>0:
@@ -130,7 +130,7 @@ class MPNN(ReaxFF):
                 bm_ = self.bom['others']
              
              if not self.TwoBodyOnly:
-                fao = tf.where(tf.greater(self.rbd[bd],self.rcuta[bd]),1.0,0.0)      ##### r> rcuta that bo = 0.0
+                fao = tf.where(tf.greater(self.rbd[bd],self.rcuta[bd]),1.0,0.0)    ##### r> rcuta that bo = 0.0
                 self.diffa[bd]  = tf.reduce_sum(input_tensor=tf.nn.relu(self.bo0[bd]*fao-self.atol))
                 l_atol = tf.add(self.diffa[bd],l_atol)
 
@@ -145,10 +145,16 @@ class MPNN(ReaxFF):
              l_atol  = tf.add(self.diffb[bd],l_atol)
              
              if self.spv_be:
-                fe  = tf.where(tf.less_equal(self.rbd[bd],self.re[bd]),1.0,0.0) ##### r< re that bo > bore_
+                fe  = tf.where(tf.less_equal(self.rbd[bd],self.re[bd]),1.0,0.0) ##### r< 0.9*re that bo > bore_
                 self.diffe[bd]  = tf.reduce_sum(input_tensor=tf.nn.relu((bore_-self.bo0[bd])*fe))
                 l_atol  = tf.add(self.diffe[bd],l_atol)
-                l_atol= tf.add(tf.nn.relu(self.rc_bo[bd]-self.rcut[bd]),l_atol)
+
+             if self.EnergyFunction == 1:
+                fe  = tf.where(tf.less_equal(self.bo0[bd],self.botol),1.0,0.0) ##### bo <= 0.0*re that e = 0.0
+                self.diffesi[bd]  = tf.reduce_sum(input_tensor=tf.nn.relu((self.EBD[bd])*fe))
+                l_atol  = tf.add(self.diffesi[bd],l_atol)
+
+             l_atol= tf.add(tf.nn.relu(self.rc_bo[bd]-self.rcut[bd]),l_atol)
       return l_atol*self.bo_penalty
 
   def get_loss(self):
@@ -224,6 +230,7 @@ class MPNN(ReaxFF):
           zpe_ = tf.gather_nd(self.EATOM,self.atomlink[mol]) 
           self.zpe[mol] = tf.reduce_sum(input_tensor=zpe_,name='zpe') + self.MolEnergy[mols]
 
+
   def get_total_energy(self):
       for mol in self.mols:
           # mols = mol.split('-')[0] 
@@ -351,12 +358,20 @@ class MPNN(ReaxFF):
       elif self.MessageFunction==2:
          Fi      = self.fmassage(flabel,bd, self.nbd[bd],[Dbi,Dbj,h],layer=self.mf_layer[1])
          Fj      = self.fmassage(flabel,bdr,self.nbd[bd],[Dbj,Dbi,h],layer=self.mf_layer[1])
-         F       = 2.0*Fi*Fj
+         F       = Fi*Fj
          Fsi,Fpi,Fpp = tf.unstack(F,axis=2)
        
          bosi    = self.Hsi[t-1][bd]*Fsi
          bopi    = self.Hpi[t-1][bd]*Fpi
          bopp    = self.Hpp[t-1][bd]*Fpp
+      elif self.MessageFunction==3:
+         Fi      = self.f_nn(flabel,bd, self.nbd[bd],[Dbi,Dbj,h],layer=self.mf_layer[1])
+         Fj      = self.f_nn(flabel,bdr,self.nbd[bd],[Dbj,Dbi,h],layer=self.mf_layer[1])
+         F       = Fi*Fj
+       
+         bosi    = self.Hsi[t-1][bd]*F
+         bopi    = self.Hpi[t-1][bd]*F
+         bopp    = self.Hpp[t-1][bd]*F
       else:
          raise NotImplementedError('-  Message function not supported yet!')
       bo      = bosi+bopi+bopp
@@ -467,20 +482,15 @@ class MPNN(ReaxFF):
       if self.EnergyFunction==3:
          self.esi[bd] = self.f_nn('fe',bd, self.nbd[bd],[self.bosi[bd],self.bopi[bd],self.bopp[bd]],
                                   layer=self.be_layer[1])
-         self.EBD[bd]  = -self.p['Desi_'+bd]*self.esi[bd]*self.bo0[bd]
+         self.EBD[bd] = -self.p['Desi_'+bd]*self.esi[bd]*self.bo0[bd]
       elif self.EnergyFunction==1:
-         Fsi  = self.f_nn('fesi',bd, self.nbd[bd],[self.bosi[bd]],layer=self.be_layer[1])  
-         Fpi  = self.f_nn('fepi',bd, self.nbd[bd],[self.bopi[bd]],layer=self.be_layer[1])  
-         Fpp  = self.f_nn('fepp',bd, self.nbd[bd],[self.bopp[bd]],layer=self.be_layer[1])  
-         #   self.esi[bd]  = self.bosi[bd]*F       
-         self.sieng[bd]= self.p['Desi_'+bd]*Fsi*self.bosi[bd]
-         self.pieng[bd]= self.p['Desi_'+bd]*Fpi*self.bopi[bd]
-         self.ppeng[bd]= self.p['Desi_'+bd]*Fpp*self.bopp[bd]
-         self.EBD[bd]  = - self.sieng[bd] - self.pieng[bd] - self.ppeng[bd]
+         self.esi[bd] = self.f_nn('fe',bd, self.nbd[bd],[self.bosi[bd],self.bopi[bd],self.bopp[bd]],
+                                  layer=self.be_layer[1])
+         self.EBD[bd] = -self.p['Desi_'+bd]*self.esi[bd] # *self.bo0[bd]
       elif self.EnergyFunction==2:
          Fi = self.f_nn('fe',bd, self.nbd[bd],[Dbi,Dbj,self.bosi[bd]],layer=self.be_layer[1])
          Fj = self.f_nn('fe',bdr,self.nbd[bd],[Dbj,Dbi,self.bosi[bd]],layer=self.be_layer[1])
-         F  = 2.0*Fi*Fj
+         F  = Fi*Fj
     
          self.esi[bd]  = self.bosi[bd]*F       
          self.sieng[bd]= tf.multiply(self.p['Desi_'+bd],self.esi[bd])
@@ -510,7 +520,7 @@ class MPNN(ReaxFF):
       reuse_m = True if self.mf_layer==self.mf_layer_ else False
       for t in range(1,self.messages+1):
           b = 0.881373587 if t>1 else -0.867
-          nout_ = 1 if self.MessageFunction==1 else 3
+          nout_ = 1 if self.MessageFunction==1 or self.MessageFunction==3 else 3
           self.set_wb(pref='f'+str(t),reuse_m=reuse_m,nin=3,nout=nout_,layer=self.mf_layer,
                       vlist=bond,nnopt=self.mpopt[t],bias=b)
 
@@ -519,16 +529,16 @@ class MPNN(ReaxFF):
       else:
          reuse_m = False 
 
-      if self.EnergyFunction==1:
-         self.set_wb(pref='fesi',reuse_m=reuse_m,nin=1,nout=1,layer=self.be_layer,
-                     vlist=self.bonds,nnopt=self.mpopt[-2],bias=2.0)
-         self.set_wb(pref='fepi',reuse_m=reuse_m,nin=1,nout=1,layer=self.be_layer,
-                     vlist=self.bonds,nnopt=self.mpopt[-2],bias=2.0)
-         self.set_wb(pref='fepp',reuse_m=reuse_m,nin=1,nout=1,layer=self.be_layer,
-                     vlist=self.bonds,nnopt=self.mpopt[-2],bias=2.0)
-      elif self.EnergyFunction>=2:
-         self.set_wb(pref='fe',reuse_m=reuse_m,nin=3,nout=1,layer=self.be_layer,
-                     vlist=self.bonds,nnopt=self.mpopt[-2],bias=2.0)
+      # if self.EnergyFunction==1:
+      #    self.set_wb(pref='fesi',reuse_m=reuse_m,nin=1,nout=1,layer=self.be_layer,
+      #                vlist=self.bonds,nnopt=self.mpopt[-2],bias=2.0)
+      #    self.set_wb(pref='fepi',reuse_m=reuse_m,nin=1,nout=1,layer=self.be_layer,
+      #                vlist=self.bonds,nnopt=self.mpopt[-2],bias=2.0)
+      #    self.set_wb(pref='fepp',reuse_m=reuse_m,nin=1,nout=1,layer=self.be_layer,
+      #                vlist=self.bonds,nnopt=self.mpopt[-2],bias=2.0)
+      # elif self.EnergyFunction>=2:
+      self.set_wb(pref='fe',reuse_m=reuse_m,nin=3,nout=1,layer=self.be_layer,
+                  vlist=self.bonds,nnopt=self.mpopt[-2],bias=2.0)
 
       if self.vdwnn:
          reuse_m = True if self.vdw_layer==self.vdw_layer_ else False
@@ -625,7 +635,10 @@ class MPNN(ReaxFF):
               'mf_layer':self.mf_layer,
               'be_layer':self.be_layer,
               'vdw_layer':self.vdw_layer,
-              'MolEnergy':self.MolEnergy_,}
+              'MolEnergy':self.MolEnergy_,
+              'rcut':self.rcut,
+              'rcutAngle':self.rcuta,
+              'rEquilibrium':self.re}
          js.dump(j,fj,sort_keys=True,indent=2)
          fj.close()
       else:
