@@ -121,13 +121,13 @@ class ReaxFF(object):
                batch_size=200,sample='uniform',
                hbshort=6.75,hblong=7.5,
                vdwcut=10.0,
-               rcut=None,rcuta=None,re=None,
                bore={'others':0.0},
                weight={'others':1.0},
                interactive=False,
                ro_scale=0.1,
                clip_op=True,
                InitCheck=True,
+               resetDeadNeuron=False,
                optmol=True,
                mefac=0.1,
                nn=False,
@@ -174,8 +174,7 @@ class ReaxFF(object):
       self.conf_vale     = conf_vale
       self.clip_op       = clip_op
       self.InitCheck     = InitCheck
-      self.rcut          = rcut
-      self.rcuta         = rcuta
+      self.resetDeadNeuron = resetDeadNeuron
       self.hbshort       = hbshort
       self.hblong        = hblong
       self.nn            = nn
@@ -201,12 +200,13 @@ class ReaxFF(object):
       self.ncpu          = ncpu
       self.bore          = bore
       self.m_,self.m     = None,None
-      self.read_lib()
-              
-      self.set_rcut(rcut,rcuta,re)
+
+      self.rcut,self.rcuta,self.re = self.read_lib()
+      self.set_rcut(self.rcut,self.rcuta,self.re)
+
       if self.InitCheck:
          self.ic = Init_Check(re=self.re,nanv=nanv)
-         self.p_ = self.ic.check(self.p_)
+         self.p_,self.m_ = self.ic.check(self.p_,self.m_,resetDeadNeuron=self.resetDeadNeuron)
 
       self.set_neurons()
       self.bo_penalty    = bo_penalty
@@ -684,7 +684,7 @@ class ReaxFF(object):
       self.Delta_e[atom] = 0.5*(D - self.p['vale_'+atom])
       self.DE[atom]      = -tf.nn.relu(-tf.math.ceil(self.Delta_e[atom])) 
 
-      self.nlp[atom] = -self.DE[atom] + tf.exp(-self.p['lp1']*4.0*tf.square(1.0+self.Delta_e[atom]-self.DE[atom]))
+      self.nlp[atom]     = -self.DE[atom] + tf.exp(-self.p['lp1']*4.0*tf.square(1.0+self.Delta_e[atom]-self.DE[atom]))
       
       self.Delta_lp[atom] = NLPOPT-self.nlp[atom]                                        # nan error
       # Delta_lp  = tf.clip_by_value(self.Delta_lp[atom],-1.0,10.0)  # temporary solution
@@ -702,7 +702,7 @@ class ReaxFF(object):
 
       self.so[atom]     = tf.gather_nd(self.SO,self.atomlist[atom])
       self.otrm1[atom]  = DIV_IF(1.0,self.Delta_lpcorr[atom]+self.p['val_'+atom])
-      # self.otrm2[atom]  = tf.math.divide(1.0,1.0+tf.exp(self.p['ovun2_'+atom]*self.Delta_lpcorr[atom]))
+      # self.otrm2[atom]= tf.math.divide(1.0,1.0+tf.exp(self.p['ovun2_'+atom]*self.Delta_lpcorr[atom]))
       self.otrm2[atom]  = tf.sigmoid(-self.p['ovun2_'+atom]*self.Delta_lpcorr[atom])
       self.EOV[atom]    = self.so[atom]*self.otrm1[atom]*self.Delta_lpcorr[atom]*self.otrm2[atom]
     
@@ -1220,13 +1220,16 @@ class ReaxFF(object):
                     'lp1','lp2','atomic']
                     
       self.lopt = ['gammaw','vdw1','rvdw','Devdw','alfa',
-                   'rohb','Dehb','hb1','hb2','atomic'] # 'gamma',
-
-      # if self.optword.find('novdw')>=0:
-      #    cons = cons + ['gamma','gammaw','vdw1','rvdw','Devdw','alfa']
+                   'rohb','Dehb','hb1','hb2','atomic']  
 
+      if self.optword.find('novdw')>=0:
+         cons = cons + ['gammaw','vdw1','rvdw','Devdw','alfa'] 
       if self.optword.find('notor')>=0:
-         cons = cons + ['tor2','tor3','tor4','V1','V2','V3','tor1','cot1','cot2'] # 
+         cons = cons + ['tor2','tor3','tor4','V1','V2','V3','tor1','cot1','cot2'] 
+      if self.optword.find('noang')>=0:
+         cons = cons + ['val1','val2','val3','val4','val5','val6','val7',
+                        'pen1','pen2','pen3','pen4',
+                        'coa1','coa2','coa3','coa4'] 
 
       if self.cons is None:
          self.cons = cons 
@@ -1252,7 +1255,7 @@ class ReaxFF(object):
           such as: val3,val6,val7,be1,lp1,ovun4,hb2,pen2
       '''   
       self.clip_pe = ['ovun3','ovun4','ovun6','ovun7',
-                      'val1','val2','val3','val4','val5',
+                      'val2','val3','val4',
                       'val6','val8','val9','val10',
                       'pen2','pen3','pen4','coa4',
                       'theta0']  
@@ -1280,7 +1283,7 @@ class ReaxFF(object):
           elif key=='ovun5':
              self.p[k] = tf.clip_by_value(self.v[k],0.0,999.0*self.unit)
           elif key=='ovun1':
-             self.p[k] = tf.clip_by_value(self.v[k],0.1,1.2)
+             self.p[k] = tf.clip_by_value(self.v[k],0.1,2.0)
           elif key in ['boc4','boc5','vdw1']:
              self.p[k] = tf.clip_by_value(self.v[k],0.00000001,100.0)
           elif key == 'boc3':
@@ -1292,9 +1295,9 @@ class ReaxFF(object):
           elif key == 'pen1':
              self.p[k] = tf.clip_by_value(self.v[k],-69.0*self.unit,199.0*self.unit)
           elif key == 'Devdw':
-             self.p[k] = tf.clip_by_value(self.v[k],0.01*self.unit,8.0*self.unit)
+             self.p[k] = tf.clip_by_value(self.v[k],0.01*self.unit,9.0*self.unit)
           elif key == 'Dehb':
-             self.p[k] = tf.clip_by_value(self.v[k],-99.0*self.unit,99.0*self.unit)
+             self.p[k] = tf.clip_by_value(self.v[k],-3.0*self.unit,3.0*self.unit)
           elif key == 'Desi':
              self.p[k] = tf.clip_by_value(self.v[k],60.000*self.unit,999.0*self.unit)
           elif key== 'Depi':
@@ -1302,7 +1305,7 @@ class ReaxFF(object):
           elif key== 'Depp':
              self.p[k] = tf.clip_by_value(self.v[k], 0.000*self.unit,999.0*self.unit)
           elif key=='tor1':
-             self.p[k] = tf.clip_by_value(self.v[k],-99.9900,-0.0001)
+             self.p[k] = tf.clip_by_value(self.v[k],-99.9900,-0.0)
           elif key== 'ovun2':
              self.p[k] = tf.clip_by_value(self.v[k],-39.00,-0.01)
           elif key in ['tor3','val7']:
@@ -1331,13 +1334,13 @@ class ReaxFF(object):
           elif key == 'boc2':
              self.p[k] = tf.clip_by_value(self.v[k],0.001,50.0)
           elif key == 'alfa':
-             self.p[k] = tf.clip_by_value(self.v[k],0.001,40.0)
+             self.p[k] = tf.clip_by_value(self.v[k],0.001,20.0)
           elif key== 'lp1':
              self.p[k] = tf.clip_by_value(self.v[k],10.0,50.0)
           elif key== 'lp2':
              self.p[k] = tf.clip_by_value(self.v[k],0.0,99.0*self.unit)
           elif key== 'lp3':
-             self.p[k] = tf.clip_by_value(self.v[k],10.0,75.0)
+             self.p[k] = tf.clip_by_value(self.v[k],10.0,75.0001)
           elif key == 'be1':
              self.p[k] = tf.clip_by_value(self.v[k],-6.0,6.0)
           elif key == 'be2':
@@ -1359,6 +1362,10 @@ class ReaxFF(object):
           elif key == 'rvdw':
              rvdw_ = 1.3*self.p_['rosi_'+vn[1]]
              self.p[k] = tf.clip_by_value(self.v[k],rvdw_,2.0*self.p_['rosi_'+vn[1]])
+          elif key=='val5':
+             self.p[k] = tf.clip_by_value(self.v[k],0.0,3.0)
+          elif key=='val1':
+             self.p[k] = tf.clip_by_value(self.v[k],0.0,99.0)
           elif key=='val':
              sp = vn[1]
              self.p[k] = tf.clip_by_value(self.v[k],0.2,8.0)
@@ -1431,7 +1438,10 @@ class ReaxFF(object):
           if self.optword.find('notor')>=0:
              if key in ktor:
                 self.p_[k] = 0.0
-
+          if self.optword.find('noang')>=0:
+             if key in ['val1','coa1','pen1']:
+                self.p_[k] = 0.0
+                
           if key == 'zpe':
              continue
           if key != 'n.u.':
@@ -1444,12 +1454,29 @@ class ReaxFF(object):
                 # with tf.variable_scope('Neurons'):
                 trainable=True if key in opt else False       
                 if key in self.punit:
-                   self.v[k] = tf.Variable(np.float32(self.unit*self.p_[k]),
-                                          trainable=trainable,name=k)
+                   if key in ['V1','V2','V3']:
+                      key = k.split('_')[1]
+                      k_  = key.split('-')
+                      if k_[1]=='H' or k_[2]=='H':
+                         self.v[k] = tf.constant(np.float32(self.unit*self.p_[k]),name=k)
+                      else:
+                         self.v[k] = tf.Variable(np.float32(self.unit*self.p_[k]),
+                                                 trainable=trainable,name=k)   
+                   elif key in ['pen1','val1']:
+                      key = k.split('_')[1]
+                      k_  = key.split('-')
+                      if k_[1]=='H':
+                         self.v[k] = tf.constant(np.float32(self.unit*self.p_[k]),name=k)
+                      else:
+                         self.v[k] = tf.Variable(np.float32(self.unit*self.p_[k]),
+                                                 trainable=trainable,name=k)  
+                   else:
+                      self.v[k] = tf.Variable(np.float32(self.unit*self.p_[k]),
+                                              trainable=trainable,name=k)
                 elif key=='vale':
                    if self.conf_vale is None:
                       self.v[k] = tf.Variable(np.float32(self.p_[k]),
-                                             trainable=trainable,name=k)
+                                              trainable=trainable,name=k)
                    else:
                       e = k.split('_')[0]
                       if e in self.conf_vale:
@@ -1619,7 +1646,7 @@ class ReaxFF(object):
 
   def reset(self,opt=[],libfile=None):
       if self.InitCheck:
-         self.p_ = self.ic.check(self.p_)
+         self.p_,self.m_ = self.ic.check(self.p_)
 
       if self.VariablesToOpt is None:
          self.set_parameters(opt=opt,libfile=libfile)
@@ -1870,11 +1897,16 @@ class ReaxFF(object):
          self.mf_layer_        = j['mf_layer']
          self.be_layer_        = j['be_layer']
          self.vdw_layer_       = j['vdw_layer']
+         rcut                  = j['rcut']
+         rcuta                 = j['rcutAngle']
+         re                    = j['rEquilibrium']
          self.init_bonds()
       else:
          self.p_,self.zpe_,self.spec,self.bonds,self.offd,self.angs,self.torp,self.hbs= \
               read_lib(libfile=self.libfile,zpe=True)
          self.MolEnergy_ = {}
+         rcut,rcuta,re = None,None,None
+      return rcut,rcuta,re
 
 
   def plot(self):
