@@ -84,6 +84,7 @@ def train_mpnn(direcs=None,step=5000,batch=None,convergence=0.97,lossConvergence
           nanv={'pen3':-2.0,'pen4':-2.0,'tor2':-5.0,'tor4':-5.0,
                 'ovun3':-2.0,'ovun4':-2.0,
                 'lp1':-3.0,'vdw1':-3.0},
+          optword='nocoul',
           spec=[],
           writelib=1000,nn=True,vdwnn=False,
           bo_layer=[3,1],
@@ -115,7 +116,7 @@ def train_mpnn(direcs=None,step=5000,batch=None,convergence=0.97,lossConvergence
                 direcs=direcs, 
                 dft='siesta',
                 spec=spec,
-                optword='nocoul',
+                optword=optword,
                 opt=opt_,
                 nn=nn,vdwnn=vdwnn,
                 bo_layer=bo_layer,
